// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_DII_H
#define OB_DII_H

namespace CORBA
{

class ExceptionList;
typedef ExceptionList* ExceptionList_ptr;
typedef ExceptionList_ptr ExceptionListRef;

class ContextList;
typedef ContextList* ContextList_ptr;
typedef ContextList_ptr ContextListRef;

class Request;
typedef Request* Request_ptr;
typedef Request_ptr RequestRef;

} // End of namespace CORBA

void OBDuplicate(CORBA::ExceptionList_ptr);
void OBRelease(CORBA::ExceptionList_ptr);
void OBDuplicate(CORBA::ContextList_ptr);
void OBRelease(CORBA::ContextList_ptr);
void OBDuplicate(CORBA::Request_ptr);
void OBRelease(CORBA::Request_ptr);

namespace CORBA
{

//
// The ExceptionList class
//
typedef OB::ObjVar< ExceptionList > ExceptionList_var;
typedef OB::ObjOut< ExceptionList > ExceptionList_out;

class ExceptionList : public OB::RefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    ExceptionList(const ExceptionList&);
    void operator=(const ExceptionList&);

    OB::TypeCodeSeq typeCodeSeq_;
    
protected:

    ExceptionList() { };
    friend class Request; // Requests may create ExceptionLists
    friend class ORB;     // ORBs create ExceptionLists

public:

    //
    // Standard IDL to C++ Mapping
    //
    static inline ExceptionList_ptr _duplicate(ExceptionList_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline ExceptionList_ptr _nil()
    { return 0; }

    ULong count() const;
    
    void add(TypeCode_ptr);
    void add_consume(TypeCode_ptr);

    TypeCode_ptr item(ULong);

    void remove(ULong);
};

inline void
release(ExceptionList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(ExceptionList_ptr p)
{
    return p == 0;
}

//
// The ContextList class
//
typedef OB::ObjVar< ContextList > ContextList_var;
typedef OB::ObjOut< ContextList > ContextList_out;

class ContextList : public OB::RefCount
{
    //
    // Hide copy-constructor and assignment operator
    //
    ContextList(const ContextList&);
    void operator=(const ContextList&);

    OB::StringSeq stringSeq_;

protected:

    ContextList() { };
    friend class Request; // Requests may create ContextLists
    friend class ORB;     // ORBs create ContextLists

public:

    //
    // Standard IDL to C++ Mapping
    //
    static inline ContextList_ptr _duplicate(ContextList_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline ContextList_ptr _nil()
    { return 0; }

    ULong count() const;
    
    void add(const char*);
    void add_consume(char*);

    const char* item(ULong);

    void remove(ULong);
};

inline void
release(ContextList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(ContextList_ptr p)
{
    return p == 0;
}

//
// The Request class
//
typedef OB::ObjVar< Request > Request_var;
typedef OB::ObjOut< Request > Request_out;

class Request : public OB::RefCount
{
    Object_var target_;
    String_var operation_;
    NVList_var arguments_;
    NamedValue_var result_;
    Environment_var environment_;
    ExceptionList_var exceptions_;
    ContextList_var contexts_;
    Context_var ctx_;
    ULong reqId_;

    //
    // Hide copy-constructor and assignment operator
    //
    Request(const Request&);
    void operator=(const Request&);

    //
    // Get client
    //
    OB::Client_ptr getClient();
    
    //
    // Marshal into sequence for a request
    // Unmarshal from sequence from a reply
    //
    void marshal(OB::Buffer&);
    void unmarshal(const OB::Buffer&, ULong, bool, bool);

protected:

    Request(Object_ptr, const char*, NVList_ptr,
	    NamedValue_ptr, ExceptionList_ptr,
	    ContextList_ptr);

    Request(Object_ptr, const char*, NVList_ptr,
	    NamedValue_ptr);

    Request(Object_ptr, const char*);
    
    friend class Object; // Objects may create Requests

public:

    //
    // Standard IDL to C++ Mapping
    //
    static inline Request_ptr _duplicate(Request_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Request_ptr _nil()
    { return 0; }

    Object_ptr target() const;
    const char* operation() const;
    NVList_ptr arguments();
    NamedValue_ptr result();
    Environment_ptr env();
    ExceptionList_ptr exceptions();
    ContextList_ptr contexts();

    void ctx(Context_ptr);
    Context_ptr ctx() const;

    Any& add_in_arg();
    Any& add_in_arg(const char*);
    Any& add_inout_arg();
    Any& add_inout_arg(const char*);
    Any& add_out_arg();
    Any& add_out_arg(const char*);
    void set_return_type(TypeCode_ptr);
    Any& return_value();

    void invoke();
    void send_oneway();
    void send_deferred();
    void get_response();
    Boolean poll_response();
};

inline void
release(Request_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(Request_ptr p)
{
    return p == 0;
}


//
// The UnknownUserException exception
// UnknownUserException defined here since it belongs logically to the
// DII
//
class UnknownUserException : public UserException
{
    Any any_;

public:
    
    //
    // Standard IDL to C++ Mapping
    //
    UnknownUserException() { }
    UnknownUserException(const UnknownUserException&);
    UnknownUserException(const Any&);
    UnknownUserException& operator=(const UnknownUserException&);
    
    Any& exception() { return any_; }

    static UnknownUserException* _downcast(Exception* p)
    { return dynamic_cast<UnknownUserException*>(p); }
    static const UnknownUserException*
    _downcast(const Exception* p)
    { return dynamic_cast<const UnknownUserException*>(p); }
    virtual void _raise() const { throw *this; }
    
    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    virtual Exception* _OB_clone() const
    { return new UnknownUserException(*this); }
    virtual void _OB_insertInAny(Any&)
    { assert_nca(false, OB::NCANoExceptionTC); }
};

} // End of namespace CORBA

#endif
